@@ -41,6 +41,10 @@ void init_cache_module(void) {
 static int cache_index(char *filename) {
 	char *c, c1, c2;
 
+	if (filename == NULL) {
+		return -1;
+	}
+
 	c = filename + strlen(filename);
 	while ((*c != '/') && (c > filename)) {
 		c--;
@@ -107,7 +111,9 @@ t_cached_object *add_to_cache(t_session *session, char *file) {
 	int fd, i;
 	size_t bytes_read, bytes_total = 0;
 
-	if (stat(file, &status) == -1) {
+	if (file == NULL) {
+		return NULL;
+	} else if (stat(file, &status) == -1) {
 		return NULL;
 	} else if ((size = status.st_size) == -1) {
 		return NULL;
@@ -180,7 +186,9 @@ t_cached_object *search_cache(t_session *session, char *file) {
 	off_t size;
 	int i;
 
-	if ((size = filesize(file)) == -1) {
+	if (file == NULL) {
+		return NULL;
+	} else if ((size = filesize(file)) == -1) {
 		return NULL;
 	}
 
