@@ -617,10 +617,11 @@ static t_cgi_result read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, c
 }
 
 t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
-	char *buffer, dummy[DUMMY_BUFFER_SIZE];
+	char *buffer, *dummy = NULL;
 	bool read_again;
 	int bytes_read, bytes_left;
 	unsigned int content, padding, data_length;
+	t_cgi_result result = cgi_OKE;
 
 	/* Read header
 	 */
@@ -649,11 +650,18 @@ t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
 		cgi_info->fcgi_data_len = 0;
 	}
 
-	/* Determine the size and type of the data
+	if (((unsigned char)cgi_info->header[1]) == FCGI_END_REQUEST) {
+		return cgi_END_OF_DATA;
+	}
+
+	/* Determine the size of the data
 	 */
 	content = 256 * (unsigned char)cgi_info->header[4] + (unsigned char)cgi_info->header[5];
 	padding = (unsigned char)cgi_info->header[6];
-	data_length = content + padding;
+	if ((data_length = content + padding) <= 0) {
+		cgi_info->read_header = true;
+		return cgi_OKE;
+	}
 
 	switch ((unsigned char)cgi_info->header[1]) {
 		case FCGI_STDOUT:
@@ -664,61 +672,61 @@ t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
 			buffer = cgi_info->error_buffer + cgi_info->error_len;
 			bytes_left = cgi_info->error_buffer_size - cgi_info->error_len;
 			break;
-		case FCGI_END_REQUEST:
-			return cgi_END_OF_DATA;
 		default:
 			/* Unsupported type, so skip data
 			 */
+			if ((dummy = (char*)malloc(DUMMY_BUFFER_SIZE)) == NULL) {
+				return cgi_ERROR;
+			}
 			buffer = dummy;
 			bytes_left = DUMMY_BUFFER_SIZE;
 	}
 
-	if (data_length > 0) {
-		/* Read data
-		 */
-		do {
-			if (bytes_left > (data_length - cgi_info->fcgi_data_len)) {
-				bytes_left = data_length - cgi_info->fcgi_data_len;
-			}
-			read_again = false;
-
-			switch (bytes_read = read_fcgi_socket(session, cgi_info, buffer, bytes_left)) {
-				case cgi_TIMEOUT:
-					return cgi_TIMEOUT;
-				case cgi_FORCE_QUIT:
-					return cgi_FORCE_QUIT;
-				case cgi_ERROR:
-					return cgi_ERROR;
-				case 0:
-					return cgi_END_OF_DATA;
-				default:
-					if ((cgi_info->fcgi_data_len += bytes_read) == data_length) {
-						cgi_info->read_header = true;
-					}
-					if (cgi_info->fcgi_data_len > content) {
-						/* Read more then content (padding)
-						 */
-						if (cgi_info->fcgi_data_len - bytes_read < content) {
-							bytes_read = content - (cgi_info->fcgi_data_len - bytes_read);
-						} else {
-							bytes_read = 0;
-						}
-					}
-					switch ((unsigned char)cgi_info->header[1]) {
-						case FCGI_STDOUT:
-							cgi_info->input_len += bytes_read;
-							break;
-						case FCGI_STDERR:
-							cgi_info->error_len += bytes_read;
-							break;
-						default:
-							break;
-					}
-			}
-		} while (read_again);
-	} else {
-		cgi_info->read_header = true;
+	/* Read data
+	 */
+	if (bytes_left > (data_length - cgi_info->fcgi_data_len)) {
+		bytes_left = data_length - cgi_info->fcgi_data_len;
 	}
 
-	return cgi_OKE;
+	switch (bytes_read = read_fcgi_socket(session, cgi_info, buffer, bytes_left)) {
+		case cgi_TIMEOUT:
+			result = cgi_TIMEOUT;
+			break;
+		case cgi_FORCE_QUIT:
+			result = cgi_FORCE_QUIT;
+			break;
+		case cgi_ERROR:
+			result = cgi_ERROR;
+			break;
+		case 0:
+			result = cgi_END_OF_DATA;
+			break;
+		default:
+			if ((cgi_info->fcgi_data_len += bytes_read) == data_length) {
+				cgi_info->read_header = true;
+			}
+			if (cgi_info->fcgi_data_len > content) {
+				/* Read more then content (padding)
+				 */
+				if (cgi_info->fcgi_data_len - bytes_read < content) {
+					bytes_read = content - (cgi_info->fcgi_data_len - bytes_read);
+				} else {
+					bytes_read = 0;
+				}
+			}
+			switch ((unsigned char)cgi_info->header[1]) {
+				case FCGI_STDOUT:
+					cgi_info->input_len += bytes_read;
+					break;
+				case FCGI_STDERR:
+					cgi_info->error_len += bytes_read;
+					break;
+				default:
+					break;
+			}
+	}
+
+	sfree(dummy);
+
+	return result;
 }
