@@ -590,13 +590,6 @@ void print_client_list(FILE *fp) {
 		client = client_list[i];
 		while (client != NULL) {
 			fprintf(fp, "  Client ID   : %d\n", client->session->client_id);
-#ifdef DEBUG
-			if (client->session->status != NULL) {
-				fprintf(fp, "  Status      : %s\n", client->session->status);
-			} else {
-				fprintf(fp, "  Status      : unknown\n");
-			}
-#endif
 			if (inet_ntop(client->session->ip_address.family, &(client->session->ip_address.value), ip_address, MAX_IP_STR_LEN) != NULL) {
 				fprintf(fp, "  IP-address  : %s\n", ip_address);
 			}
