@@ -82,7 +82,7 @@ static int add_to_environment_chroot(t_session *session, t_fcgi_buffer *fcgi_buf
 
 /* Copy a headerfield to an environment setting
  */
-static int headerfield_to_environment(t_session *session, t_fcgi_buffer *fcgi_buffer, char *key, char *envir) {
+int headerfield_to_environment(t_session *session, t_fcgi_buffer *fcgi_buffer, char *key, char *envir) {
 	char *value;
 
 	if ((key == NULL) || (envir == NULL)) {
@@ -97,8 +97,9 @@ static int headerfield_to_environment(t_session *session, t_fcgi_buffer *fcgi_bu
 /* Set environment variables for CGI script.
  */
 void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
-	char ip[MAX_IP_STR_LEN], len[20], value[10], *data, variable[MAX_HEADER_LEN];
-	size_t len1, len2;
+	char ip[MAX_IP_STR_LEN], len[20], value[10], *data, variable[MAX_HEADER_LEN], old;
+	size_t len1, len2, path_info_len;
+	bool has_path_info = false;
 	t_headerfield *headerfields;
 	t_keyvalue *envir;
 #ifdef HAVE_SSL
@@ -112,13 +113,29 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 
 	add_to_environment(fcgi_buffer, "REQUEST_METHOD", session->method);
 	add_to_environment(fcgi_buffer, "REQUEST_URI", session->request_uri);
-	if (session->uri[session->uri_len - 1] == '/') {
+
+	if (session->path_info != NULL) {
+		path_info_len = strlen(session->path_info);
+		if (path_info_len <= session->uri_len) {
+			if (memcmp(session->uri + session->uri_len - path_info_len, session->path_info, path_info_len) == 0) {
+				has_path_info = true;
+			}
+		}
+	}
+
+	if (has_path_info) {
+		old = *(session->uri + session->uri_len - path_info_len);
+		*(session->uri + session->uri_len - path_info_len) = '\0';
+		add_to_environment(fcgi_buffer, "SCRIPT_NAME", session->uri);
+		*(session->uri + session->uri_len - path_info_len) = old;
+	} else if (session->uri[session->uri_len - 1] == '/') {
 		len1 = session->uri_len;
 		len2 = strlen(session->host->start_file);
+
 		if ((data = (char*)malloc(len1 + len2 + 1)) != NULL) {
 			memcpy(data, session->uri, len1);
 			memcpy(data + len1, session->host->start_file, len2);
-			data[len1 + len2] = '\0';
+			*(data + len1 + len2) = '\0';
 			add_to_environment(fcgi_buffer, "SCRIPT_NAME", data);
 			free(data);
 		}
