@@ -110,25 +110,6 @@ char *version_string = "Hiawatha v"VERSION
 #endif
 ;
 
-#ifdef DEBUG
-/* timer functions for debugging
- */
-double t1, t2;
-struct timeval tp;
-
-void start_clock(char *mesg) {
-	fprintf(stderr, "(%5d) START: %s\n", (int)getpid(), mesg);
-	gettimeofday(&tp, NULL);
-	t1 = (double)tp.tv_sec + (1.e-6)*tp.tv_usec;
-}
-
-void stop_clock(void) {
-	gettimeofday(&tp, NULL);
-	t2 = (double)tp.tv_sec + (1.e-6)*tp.tv_usec;
-	fprintf(stderr, "(%5d)  time: %f\n", (int)getpid(), t2 - t1);
-}
-#endif
-
 /* Create all logfiles with the right ownership and accessrights
  */
 void touch_logfiles(t_config *config, char *dir) {
@@ -138,6 +119,9 @@ void touch_logfiles(t_config *config, char *dir) {
 	if (config->garbage_logfile != NULL) {
 		touch_logfile(dir, config->garbage_logfile, 0640, config->server_uid, config->server_gid);
 	}
+#ifdef DEBUG
+	touch_logfile(dir, LOG_DIR"/debug.log", 0640, config->server_uid, config->server_gid);
+#endif
 
 	host = config->first_host;
 	while (host != NULL) {
@@ -159,6 +143,7 @@ t_cgi_type check_target_is_cgi(t_session *session) {
 #ifdef HAVE_TOOLKIT
 	if ((session->fcgi_server = find_fcgi_server(session->config->fcgi_server, session->toolkit_fastcgi)) != NULL) {
 		session->cgi_type = fastcgi;
+		session->host->execute_cgi = true;
 	} else 
 #endif
 	if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->extension)) != NULL) {
@@ -184,7 +169,7 @@ t_cgi_type check_target_is_cgi(t_session *session) {
 /* Handle an HTTP error.
  */
 int handle_error(t_session *session, int error_code) {
-	char *error_handler = NULL;
+	char *error_handler = NULL, *new_fod;
 	int result = -1;
 
 	switch (error_code) {
@@ -212,12 +197,14 @@ int handle_error(t_session *session, int error_code) {
 	session->error_code = error_code;
 	session->handling_error = true;
 	session->mimetype = NULL;
-	if (session->file_on_disk != NULL) {
-		free(session->file_on_disk);
-	}
-	if ((session->file_on_disk = (char*)malloc(session->host->website_root_len + strlen(error_handler) + 4)) == NULL) { /* + 3 for .gz (gzip encoding) */
+	if ((new_fod = (char*)malloc(session->host->website_root_len + strlen(error_handler) + 4)) == NULL) { /* + 3 for .gz (gzip encoding) */
 		result = 500;
 	} else {
+		if (session->file_on_disk != NULL) {
+			free(session->file_on_disk);
+		}
+		session->file_on_disk = new_fod;
+
 		memcpy(session->file_on_disk, session->host->website_root, session->host->website_root_len);
 		strcpy(session->file_on_disk + session->host->website_root_len, error_handler);
 
@@ -305,6 +292,9 @@ int run_program(t_session *session, char *program) {
 				if (inet_ntop(session->ip_address.family, &(session->ip_address.value), ip, MAX_IP_STR_LEN) != NULL) {
 					setenv("REMOTE_ADDR", ip, 1);
 				}
+				headerfield_to_environment(session, NULL, "Range:", "HTTP_RANGE");
+				headerfield_to_environment(session, NULL, "Referer:", "HTTP_REFERER");
+				headerfield_to_environment(session, NULL, "User-Agent:", "HTTP_USER_AGENT");
 
 				/* Change directory to program's directory
 				 */
@@ -354,9 +344,6 @@ int fetch_request(t_session *session) {
 		deadline = time(NULL) + session->binding->time_for_request;
 	}
 
-#ifdef DEBUG
-	set_client_status(session, "fetch_request()");
-#endif
 	do {
 		/* Check if requestbuffer contains a complete request.
 		 */
@@ -463,9 +450,6 @@ int fetch_request(t_session *session) {
 			}
 		}
 
-#ifdef DEBUG
-		set_client_status(session, "fetch_request()::select()");
-#endif
 		FD_ZERO(&read_fds);
 		FD_SET(session->client_socket, &read_fds);
 
@@ -501,9 +485,6 @@ int fetch_request(t_session *session) {
 
 				/* Read from socket.
 				 */
-#ifdef DEBUG
-				set_client_status(session, "fetch_request()::recv()");
-#endif
 #ifdef HAVE_SSL
 				if (session->binding->use_ssl) {
 					bytes_read = ssl_receive(session->ssl_data, session->request + session->bytes_in_buffer, 
@@ -570,10 +551,6 @@ int fetch_request(t_session *session) {
 	increment_transfer(TRANSFER_RECEIVED, header_length + content_length);
 #endif
 
-#ifdef DEBUG
-	set_client_status(session, NULL);
-#endif
-
 	return result;
 }
 
@@ -697,18 +674,14 @@ int uri_to_path(t_session *session) {
 				memcpy(session->local_user, strstart, length);
 				*(session->local_user + length) = '\0';
 				
-				if (duplicate_host(session)) {
-					if ((retval = get_homedir(session, session->local_user + 1)) != 200) {
-						return retval;
-					}
-					session->host->error_401_handler = NULL;
-					session->host->error_403_handler = NULL;
-					session->host->error_404_handler = NULL;
-					session->host->error_501_handler = NULL;
-					session->host->error_503_handler = NULL;
-				} else {
-					return 500;
+				if ((retval = get_homedir(session, session->local_user + 1)) != 200) {
+					return retval;
 				}
+				session->host->error_401_handler = NULL;
+				session->host->error_403_handler = NULL;
+				session->host->error_404_handler = NULL;
+				session->host->error_501_handler = NULL;
+				session->host->error_503_handler = NULL;
 			} else {
 				/* uri is '/~/...' */
 				return 404;
@@ -755,6 +728,7 @@ int uri_to_path(t_session *session) {
 		length = strlen(alias->value);
 		memcpy(session->file_on_disk, alias->value, length);
 		strstart = session->uri + alias_length;
+
 	}
 	strcpy(session->file_on_disk + length, strstart);
 
@@ -880,9 +854,6 @@ int serve_client(t_session *session) {
 				return 500;
 			}
 			if (new_uri != NULL) {
-#ifdef DEBUG
-				fprintf(stderr, "Rewrite URL: [%s] --> [%s]\n", session->uri, new_uri);
-#endif
 				if (register_tempdata(&(session->tempdata), new_uri, tc_data) == -1) {
 					free(new_uri);
 					return 500;
@@ -912,12 +883,17 @@ int serve_client(t_session *session) {
 
 	url_decode(session->uri);
 	session->uri_len = strlen(session->uri);
+
 	if ((session->vars != NULL) && (session->host->secure_url)) {
 		if (forbidden_chars_present(session->vars)) {
 			return 403;
 		}
 	}
 
+	if (duplicate_host(session) == false) {
+		return 500;
+	}
+
 	if (valid_uri(session->uri) == false) {
 		if (session->request_method == PUT) {
 			return 403;
@@ -933,20 +909,16 @@ int serve_client(t_session *session) {
 	while (*search != '\0') {
 		if (*search == '/') {
 			length = search - session->file_on_disk + 1;
-			if ((conffile = (char*)malloc(length + 10)) != NULL) {
+			if ((conffile = (char*)malloc(length + 10)) == NULL) {
+				result = 500;
+			} else {
 				memcpy(conffile, session->file_on_disk, length);
 				memcpy(conffile + length, ".hiawatha\0", 10);
-				if (duplicate_host(session)) {
-					if (read_user_configfile(conffile, session->host, &(session->tempdata)) > 0) {
-						log_file_error(session, conffile, "error in configuration file");
-						result = 500;
-					}
-				} else {
+				if (read_user_configfile(conffile, session->host, &(session->tempdata)) > 0) {
+					log_file_error(session, conffile, "error in configuration file");
 					result = 500;
 				}
 				free(conffile);
-			} else {
-				result = 500;
 			}
 		}
 		if (result == 200) {
@@ -1092,12 +1064,6 @@ int serve_client(t_session *session) {
 /* Request has been handled, handle the return code.
  */
 void handle_request_result(t_session *session, int result) {
-	if ((result > 0) && (result != 400)) {
-		log_request(session, result);
-	} else {
-		session->keep_alive = false;
-	}
-
 	switch (result) {
 		case fe_MAX_REQUESTSIZE:
 			log_system(session, "Maximum request size reached");
@@ -1126,29 +1092,26 @@ void handle_request_result(t_session *session, int result) {
 		case fe_READ_ERROR:
 			if (errno != ECONNRESET) {
 				log_system(session, "Error while reading request");
-#ifdef DEBUG
-				log_system(session, strerror(errno));
-#endif
 			}
 			break;
 		case fe_FORCE_QUIT:
 			log_system(session, "Client kicked");
 			break;
 		case rr_CMD_INJECTION:
-			log_request(session, 400);
 			if (ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_cmdi, session->config->kick_on_ban);
 				log_system(session, "Client banned because of command injection");
 			}
 			send_code(session, 400);
+			log_request(session, 400);
 			break;
 		case rr_SQL_INJECTION:
-			log_request(session, 400);
 			if (ip_allowed(&(session->ip_address), session->config->banlist_mask)) {
 				ban_ip(&(session->ip_address), session->config->ban_on_sqli, session->config->kick_on_ban);
 				log_system(session, "Client banned because of SQL injection");
 			}
 			send_code(session, 400);
+			log_request(session, 400);
 			break;
 		case 200:
 			break;
@@ -1209,6 +1172,12 @@ void handle_request_result(t_session *session, int result) {
 				send_code(session, result);
 			}
 	}
+
+	if ((result > 0) && (result != 400)) {
+		log_request(session, result);
+	} else {
+		session->keep_alive = false;
+	}
 }
 
 /* Handle the connection of a client.
@@ -1482,11 +1451,17 @@ int accept_connection(t_binding *binding, t_config *config) {
 	pthread_attr_t      child_attr;
 	pthread_t           child_thread;
 	int                 total_conns, one;
-struct timeval timer;
+	struct timeval      timer;
+#ifdef DEBUG
+	static int          thread_id = 0;
+#endif
 
 	if ((session = (t_session*)malloc(sizeof(t_session))) == NULL) {
 		return -1;
 	}
+#ifdef DEBUG
+	session->thread_id = thread_id++;
+#endif
 	session->config = config;
 	session->binding = binding;
 	init_session(session);
@@ -1498,10 +1473,11 @@ struct timeval timer;
 		memset((void*)&caddr4, 0, (size_t)size);
 		if ((session->client_socket = accept(binding->socket, (struct sockaddr*)&caddr4, &size)) == -1) {
 			free(session);
-			if (errno != EINTR) {
-				return -1;
+			log_string(config->system_logfile, "Error accepting incoming IPv4 connection: %s", strerror(errno));
+			if (errno == EINTR) {
+				return 0;
 			}
-			return 0;
+			return -1;
 		}
 
 		session->ip_address.family = AF_INET;
@@ -1515,10 +1491,11 @@ struct timeval timer;
 		memset((void*)&caddr6, 0, (size_t)size);
 		if ((session->client_socket = accept(binding->socket, (struct sockaddr*)&caddr6, &size)) == -1) {
 			free(session);
-			if (errno != EINTR) {
-				return -1;
+			log_string(config->system_logfile, "Error accepting incoming IPv6 connection: %s", strerror(errno));
+			if (errno == EINTR) {
+				return 0;
 			}
-			return 0;
+			return -1;
 		}
 
 		session->ip_address.family = AF_INET6;
@@ -1538,6 +1515,7 @@ struct timeval timer;
 			if (setsockopt(session->client_socket, IPPROTO_TCP, TCP_NODELAY, (void*)&one, sizeof(int)) == -1) {
 				close(session->client_socket);
 				free(session);
+				log_string(config->system_logfile, "error setsockopt(TCP_NODELAY)");
 				return -1;
 			}
 
@@ -1547,6 +1525,7 @@ struct timeval timer;
 				if (setsockopt(session->client_socket, SOL_SOCKET, SO_SNDTIMEO, &timer, sizeof(struct timeval)) == -1) {
 					close(session->client_socket);
 					free(session);
+					log_string(config->system_logfile, "error setsockopt(SO_SNDTIMEO)");
 					return -1;
 				}
 			}
@@ -1840,7 +1819,6 @@ int run_server(t_settings *settings) {
 			fprintf(stderr, "Error redirecting stdin\n");
 			return EXIT_FAILURE;
 		}
-#ifndef DEBUG
 		if (close(STDOUT_FILENO) == -1) {
 			fprintf(stderr, "Warning: error closing STDOUT\n");
 		} else if (open("/dev/null", O_WRONLY) == -1) {
@@ -1853,7 +1831,6 @@ int run_server(t_settings *settings) {
 			log_string(config->system_logfile, "Error redirecting stderr\n");
 			return EXIT_FAILURE;
 		}
-#endif
 	}
 
 	log_string(config->system_logfile, "Hiawatha v"VERSION" started");
@@ -1914,7 +1891,6 @@ int run_server(t_settings *settings) {
 				while (binding != NULL) {
 					if (FD_ISSET(binding->socket, &read_fds)) {
 						if (accept_connection(binding, config) != 0) {
-							log_string(config->system_logfile, "Fatal error accepting connection");
 							usleep(1000);
 							break;
 
