@@ -1,9 +1,11 @@
 #include "config.h"
+#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
 #include <string.h>
 #include <limits.h>
+#include "global.h"
 #include "alternative.h"
 
 #define MAX_VALUE (INT_MAX - 10) / 10
@@ -66,15 +68,22 @@ char *remove_spaces(char *str) {
 char *uncomment(char *str) {
 	char *hash;
 
-	if (str != NULL) {
-		if ((hash = strchr(str, '#')) != NULL) {
-			*hash = '\0';
-		}
-
-		return remove_spaces(str);
+	if (str == NULL) {
+		return NULL;
 	}
 
-	return NULL;
+	if (*str == '#') {
+		*str = '\0';
+		return str;
+	}
+
+	if ((hash = strstr(str, " #")) != NULL) {
+		*hash = '\0';
+	} else if ((hash = strstr(str, "\t#")) != NULL) {
+		*hash = '\0';
+	}
+
+	return remove_spaces(str);
 }
 
 /* Covert a string to lowercase.
@@ -147,6 +156,10 @@ int split_configline(const char *str, char **key, char **value) {
 /* Check the validity of an URL.
  */
 bool valid_uri(char *uri) {
+#ifdef CYGWIN
+	size_t last_pos;
+#endif
+
 	if (uri == NULL) {
 		return false;
 	} else if (*uri != '/') {
@@ -154,7 +167,10 @@ bool valid_uri(char *uri) {
 	}
 
 #ifdef CYGWIN
-	if (*(uri + strlen(uri) - 1) == '.') {
+	last_pos = strlen(uri) - 1;
+	if (*(uri + last_pos) == '.') {
+		return false;
+	} else if (*(uri + last_pos) == ' ') {
 		return false;
 	}
 
@@ -193,7 +209,7 @@ static short hex_to_int(char c) {
 /* Encode a string to an URL encoded one
  */
 static bool char_needs_encoding(char c) {
-	return (c <= 32) || (strchr("\"#%'<>", c) != NULL) || (c >= 126);
+	return (c <= 32) || (strchr("\"#\%&'+:<>", c) != NULL) || (c >= 126);
 }
 int url_encode(char *str, char **encoded) {
 	char *c, *e;
@@ -646,3 +662,44 @@ int header_to_variable(char *header, char *variable, int size) {
 
 	return 0;
 }
+
+/* Converts a filesize to a string.
+ */
+int filesize2str(char *buffer, int len, off_t fsize) {
+	int result = 0;
+
+	buffer[len - 1] = '\0';
+	if (fsize < KILOBYTE) {
+		result = snprintf(buffer, len - 1, "%llu byte", (long long)fsize);
+	} else if (fsize < MEGABYTE) {
+		result = snprintf(buffer, len - 1, "%0.1f kB", ((double)(fsize >> 6)) / 16);
+	} else if (fsize < GIGABYTE) {
+		result = snprintf(buffer, len - 1, "%0.1f MB", ((double)(fsize >> 16)) / 16);
+	} else {
+		result = snprintf(buffer, len - 1, "%0.1f GB", ((double)(fsize >> 26)) / 16);
+	}
+
+	return (result < 0) ? 0 : result;
+}
+
+
+int add_str(char **buffer, int *size, int extra_size, int *len, char *str) {
+	size_t str_len;
+	char *new;
+
+	str_len = strlen(str);
+	while (*len + str_len >= *size) {
+		*size += extra_size;
+		if ((new = (char*)realloc(*buffer, *size)) == NULL) {
+			*size -= extra_size;
+			return -1;
+		}
+		*buffer = new;
+	}
+
+	memcpy(*buffer + *len, str, str_len);
+	*len += str_len;
+	*(*buffer + *len) = '\0';
+
+	return 0;
+}
