@@ -7,6 +7,7 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include <stdbool.h>
+#include <stdarg.h>
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
@@ -96,15 +97,26 @@ void log_pid(t_config *config, pid_t pid) {
 
 /* Log a text.
  */
-void log_string(char *logfile, char *mesg) {
+void log_string(char *logfile, char *mesg, ...) {
 	FILE *fp;
+	va_list args;
 	char str[TIMESTAMP_SIZE];
 
-	if ((fp = fopen(logfile, "a")) != NULL) {
-		print_timestamp(str);
-		fprintf(fp, "%s%s"EOL, str, mesg);
-		fclose(fp);
+	if (mesg == NULL) {
+		return;
+	} else if ((fp = fopen(logfile, "a")) == NULL) {
+		return;
 	}
+
+	va_start(args, mesg);
+
+	print_timestamp(str);
+	fprintf(fp, "%s", str);
+	vfprintf(fp, mesg, args);
+	fprintf(fp, EOL);
+	fclose(fp);
+
+	va_end(args);
 }
 
 /* Log a system message.
@@ -113,12 +125,16 @@ void log_system(t_session *session, char *mesg) {
 	FILE *fp;
 	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
-	if ((fp = fopen(session->config->system_logfile, "a")) != NULL) {
-		print_ip_address(str, &(session->ip_address));
-		print_timestamp(str + strlen(str));
-		fprintf(fp, "%s%s\n", str, mesg);
-		fclose(fp);
+	if (mesg == NULL) {
+		return;
+	} else if ((fp = fopen(session->config->system_logfile, "a")) != NULL) {
+		return;
 	}
+
+	print_ip_address(str, &(session->ip_address));
+	print_timestamp(str + strlen(str));
+	fprintf(fp, "%s%s\n", str, mesg);
+	fclose(fp);
 }
 
 /* Log an error for a specific file
@@ -127,16 +143,20 @@ void log_file_error(t_session *session, char *file, char *mesg) {
 	FILE *fp;
 	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
-	if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
-		print_timestamp(str);
-		print_ip_address(str + strlen(str), &(session->ip_address));
-		if (file != NULL) {
-			fprintf(fp, "%s%s|%s"EOL, str, file, mesg);
-		} else {
-			fprintf(fp, "%s-|%s"EOL, str, mesg);
-		}
-		fclose(fp);
+	if ((file == NULL) || (mesg == NULL)) {
+		return;
+	} else if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
+		return;
 	}
+
+	print_timestamp(str);
+	print_ip_address(str + strlen(str), &(session->ip_address));
+	if (file != NULL) {
+		fprintf(fp, "%s%s|%s"EOL, str, file, mesg);
+	} else {
+		fprintf(fp, "%s-|%s"EOL, str, mesg);
+	}
+	fclose(fp);
 }
 
 /* Log an error
@@ -155,132 +175,133 @@ void log_request(t_session *session, int code) {
 	time_t t;
 	struct tm *s;
 
-	if (ip_allowed(&(session->ip_address), session->config->logfile_mask) != deny) {
-		str[BUFFER_SIZE] = '\0';
-
-#ifdef HAVE_TOOLKIT
-		if (session->request_uri == NULL) {
-#endif
-			uri = secure_string(session->uri);
-			path_info = secure_string(session->path_info);
-			vars = secure_string(session->vars);
-#ifdef HAVE_TOOLKIT
-
-		} else {
-			uri = secure_string(session->request_uri);
-			path_info = NULL;
-			vars = NULL;
-		}
-#endif
-
-		if ((user = session->remote_user) != NULL) {
-			user = secure_string(user);
-		}
-
-		if (session->config->log_format == hiawatha) {
-			/* Hiawatha log format
-			 */
-			print_ip_address(str, &(session->ip_address));
-			offset = strlen(str);
-			print_timestamp(str + offset);
-			offset += strlen(str + offset);
-
-			if (user == NULL) {
-				user = "";
-			}
-
-			snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, user, secure_string(session->method), uri);
-			offset += strlen(str + offset);
-
-			if ((offset < BUFFER_SIZE) && (path_info != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "/%s", path_info);
-				offset += strlen(str + offset);
-			}
-			if ((offset < BUFFER_SIZE) && (vars != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", vars);
-				offset += strlen(str + offset);
-			}
-
-			if (offset < BUFFER_SIZE) {
-				snprintf(str + offset, BUFFER_SIZE - offset, " %s", secure_string(session->http_version));
-				offset += strlen(str + offset);
-			}
-			
-			if (offset < BUFFER_SIZE) {
-				headerfield = session->headerfields;
-				while (headerfield != NULL) {
-					if ((strncasecmp("Cookie:", headerfield->data, 7) != 0) && (strncasecmp("Authorization:", headerfield->data, 14) != 0)) {
-						snprintf(str + offset, BUFFER_SIZE - offset, "|%s", secure_string(headerfield->data));
-						if ((offset += strlen(str + offset)) >= BUFFER_SIZE) {
-							break;
-						}
-					}
-					headerfield = headerfield->next;
-				}
-			}
-		} else {
-			/* Common Log Format
-			 */
-			print_ip_address(ip_address, &(session->ip_address));
-			if ((len = strlen(ip_address)) > 0) {
-				ip_address[len - 1] = '\0';
-			}
-
-			if (user == NULL) {
-				user = "-";
-			}
-
-			time(&t);
-			s = localtime(&t);
-			timestamp[TIMESTAMP_SIZE - 1] = '\0';
-			strftime(timestamp, TIMESTAMP_SIZE - 1, "%d/%b/%Y:%T %z", s);
-
-			snprintf(str, BUFFER_SIZE, "%s - %s [%s] \"%s %s", ip_address, user, timestamp, secure_string(session->method), uri);
-			offset = strlen(str);
-			if ((offset < BUFFER_SIZE) && (path_info != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "/%s", path_info);
-				offset += strlen(str + offset);
-			}
-			if ((offset < BUFFER_SIZE) && (vars != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", vars);
-				offset += strlen(str + offset);
-			}
-			if (offset < BUFFER_SIZE) {
-				snprintf(str + offset, BUFFER_SIZE - offset, " %s\" %d %lld", secure_string(session->http_version), code, (long long)session->bytes_sent);
-			}
-
-			if (session->config->log_format == extended) {
-				/* Extended Common Log Format
-			 	 */
-				offset += strlen(str + offset);
-				if (offset < BUFFER_SIZE) {
-					if ((field = get_headerfield("Referer:", session->headerfields)) != NULL) {
-						snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", secure_string(field));
-					} else {
-						snprintf(str + offset, BUFFER_SIZE - offset, " \"-\"");
-					}
-					offset += strlen(str + offset);
-				}
-				if (offset < BUFFER_SIZE) {
-					if ((field = get_headerfield("User-Agent:", session->headerfields)) != NULL) {
-						snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", secure_string(field));
-					} else {
-						snprintf(str + offset, BUFFER_SIZE - offset, " \"-\"");
-					}
-				}
-			}
-		}
-
-		pthread_mutex_lock(&accesslog_mutex);
-		if (*(session->host->access_fp) == NULL) {
-			*(session->host->access_fp) = fopen(session->host->access_logfile, "a");
-		}
-		if (*(session->host->access_fp) != NULL) {
-			fprintf(*(session->host->access_fp), "%s"EOL, str);
-			fflush(*(session->host->access_fp));
-		}
-		pthread_mutex_unlock(&accesslog_mutex);
+	if (ip_allowed(&(session->ip_address), session->config->logfile_mask) == deny) {
+		return;
 	}
+
+	str[BUFFER_SIZE] = '\0';
+
+#ifdef HAVE_TOOLKIT
+	if (session->request_uri == NULL) {
+#endif
+		uri = secure_string(session->uri);
+		path_info = secure_string(session->path_info);
+		vars = secure_string(session->vars);
+#ifdef HAVE_TOOLKIT
+	} else {
+		uri = secure_string(session->request_uri);
+		path_info = NULL;
+		vars = NULL;
+	}
+#endif
+
+	if ((user = session->remote_user) != NULL) {
+		user = secure_string(user);
+	}
+
+	if (session->config->log_format == hiawatha) {
+		/* Hiawatha log format
+		 */
+		print_ip_address(str, &(session->ip_address));
+		offset = strlen(str);
+		print_timestamp(str + offset);
+		offset += strlen(str + offset);
+
+		if (user == NULL) {
+			user = "";
+		}
+
+		snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, user, secure_string(session->method), uri);
+		offset += strlen(str + offset);
+
+		if ((offset < BUFFER_SIZE) && (path_info != NULL)) {
+			snprintf(str + offset, BUFFER_SIZE - offset, "/%s", path_info);
+			offset += strlen(str + offset);
+		}
+		if ((offset < BUFFER_SIZE) && (vars != NULL)) {
+			snprintf(str + offset, BUFFER_SIZE - offset, "?%s", vars);
+			offset += strlen(str + offset);
+		}
+
+		if (offset < BUFFER_SIZE) {
+			snprintf(str + offset, BUFFER_SIZE - offset, " %s", secure_string(session->http_version));
+			offset += strlen(str + offset);
+		}
+		
+		if (offset < BUFFER_SIZE) {
+			headerfield = session->headerfields;
+			while (headerfield != NULL) {
+				if ((strncasecmp("Cookie:", headerfield->data, 7) != 0) && (strncasecmp("Authorization:", headerfield->data, 14) != 0)) {
+					snprintf(str + offset, BUFFER_SIZE - offset, "|%s", secure_string(headerfield->data));
+					if ((offset += strlen(str + offset)) >= BUFFER_SIZE) {
+						break;
+					}
+				}
+				headerfield = headerfield->next;
+			}
+		}
+	} else {
+		/* Common Log Format
+		 */
+		print_ip_address(ip_address, &(session->ip_address));
+		if ((len = strlen(ip_address)) > 0) {
+			ip_address[len - 1] = '\0';
+		}
+
+		if (user == NULL) {
+			user = "-";
+		}
+
+		time(&t);
+		s = localtime(&t);
+		timestamp[TIMESTAMP_SIZE - 1] = '\0';
+		strftime(timestamp, TIMESTAMP_SIZE - 1, "%d/%b/%Y:%T %z", s);
+
+		snprintf(str, BUFFER_SIZE, "%s - %s [%s] \"%s %s", ip_address, user, timestamp, secure_string(session->method), uri);
+		offset = strlen(str);
+		if ((offset < BUFFER_SIZE) && (path_info != NULL)) {
+			snprintf(str + offset, BUFFER_SIZE - offset, "/%s", path_info);
+			offset += strlen(str + offset);
+		}
+		if ((offset < BUFFER_SIZE) && (vars != NULL)) {
+			snprintf(str + offset, BUFFER_SIZE - offset, "?%s", vars);
+			offset += strlen(str + offset);
+		}
+		if (offset < BUFFER_SIZE) {
+			snprintf(str + offset, BUFFER_SIZE - offset, " %s\" %d %lld", secure_string(session->http_version), code, (long long)session->bytes_sent);
+		}
+
+		if (session->config->log_format == extended) {
+			/* Extended Common Log Format
+			 */
+			offset += strlen(str + offset);
+			if (offset < BUFFER_SIZE) {
+				if ((field = get_headerfield("Referer:", session->headerfields)) != NULL) {
+					snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", secure_string(field));
+				} else {
+					snprintf(str + offset, BUFFER_SIZE - offset, " \"-\"");
+				}
+				offset += strlen(str + offset);
+			}
+			if (offset < BUFFER_SIZE) {
+				if ((field = get_headerfield("User-Agent:", session->headerfields)) != NULL) {
+					snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", secure_string(field));
+				} else {
+					snprintf(str + offset, BUFFER_SIZE - offset, " \"-\"");
+				}
+			}
+		}
+	}
+
+	pthread_mutex_lock(&accesslog_mutex);
+	if (*(session->host->access_fp) == NULL) {
+		*(session->host->access_fp) = fopen(session->host->access_logfile, "a");
+	}
+	if (*(session->host->access_fp) != NULL) {
+		fprintf(*(session->host->access_fp), "%s"EOL, str);
+		fflush(*(session->host->access_fp));
+	}
+	pthread_mutex_unlock(&accesslog_mutex);
 }
 
 /* Log garbage sent by a client.
@@ -317,12 +338,16 @@ void log_unban(char *logfile, t_ip_addr *ip_address, unsigned long connect_attem
 	FILE *fp;
 	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
-	if ((fp = fopen(logfile, "a")) != NULL) {
-		print_ip_address(str, ip_address);
-		print_timestamp(str + strlen(str));
-		fprintf(fp, "%sUnbanned (%ld connect attempts during ban)\n", str, connect_attempts);
-		fclose(fp);
+	if ((logfile == NULL) || (ip_address == NULL)) {
+		return;
+	} else if ((fp = fopen(logfile, "a")) == NULL) {
+		return;
 	}
+
+	print_ip_address(str, ip_address);
+	print_timestamp(str + strlen(str));
+	fprintf(fp, "%sUnbanned (%ld connect attempts during ban)\n", str, connect_attempts);
+	fclose(fp);
 }
 
 /* Log a CGI error.
@@ -332,6 +357,10 @@ void log_cgi_error(t_session *session, char *mesg) {
 	char *c, str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 	int len = 0;
 
+	if (mesg == NULL) {
+		return;
+	}
+
 	c = mesg;
 	while (*c != '\0') {
 		if (*c == '\n') {
@@ -391,3 +420,28 @@ void close_logfiles_for_cgi_run(t_host *host) {
 		host = host->next;
 	}
 }
+
+#ifdef DEBUG
+void log_debug(t_session *session, char *mesg, ...) {
+	FILE *fp;
+	va_list args;
+	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
+
+	if (mesg == NULL) {
+		return;
+	} else if ((fp = fopen(LOG_DIR"/access.log", "a")) == NULL) {
+		return;
+	}
+
+	va_start(args, mesg);
+
+	print_ip_address(str, &(session->ip_address));
+	print_timestamp(str + strlen(str));
+	fprintf(fp, "%s%05d|", str, session->thread_id);
+	vfprintf(fp, mesg, args);
+	fprintf(fp, EOL);
+	fclose(fp);
+
+	va_end(args);
+}
+#endif
