@@ -37,6 +37,7 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 
 	while (fgets(line, MAX_LINE_LENGTH, fp) != NULL) {
 		if ((next = data = strdup(line)) == NULL) {
+			fclose(fp);
 			return -1;
 		}
 		while (*next != '\0') {
@@ -51,10 +52,12 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 			*next++ = '\0';
 			if (w_mime == NULL) {
 				if (((*mime)->mimename = w_mime = (t_mimename*)malloc(sizeof(t_mimename))) == NULL) {
+					fclose(fp);
 					return -1;
 				}
 			} else {
 				if ((w_mime->next = (t_mimename*)malloc(sizeof(t_mimename))) == NULL) {
+					fclose(fp);
 					return -1;
 				}
 				w_mime = w_mime->next;
@@ -77,10 +80,12 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 					if (strlen(ext) <= MAX_EXT) {
 						if (w_ext == NULL) {
 							if (((*mime)->extension = w_ext = (t_extension*)malloc(sizeof(t_extension))) == NULL) {
+								fclose(fp);
 								return -1;
 							}
 						} else {
 							if ((w_ext->next = (t_extension*)malloc(sizeof(t_extension))) == NULL) {
+								fclose(fp);
 								return -1;
 							}
 							w_ext = w_ext->next;
@@ -96,6 +101,7 @@ int read_mimetypes(char *configfile, t_mimetype **mime) {
 			}
 		}
 	}
+
 	fclose(fp);
 
 	return 0;
@@ -109,7 +115,7 @@ char *get_mimetype(char *extension, t_mimetype *mime) {
 
 	/* Convert file extension to mimetype.
 	 */ 
-	if (extension == NULL) {
+	if ((extension == NULL) || (mime == NULL)) {
 		return NULL;
 	} else if ((ext_lower = strlower(strdup(extension))) == NULL) {
 		return NULL;
