@@ -86,10 +86,6 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 			can_send = rest;
 		}
 
-#ifdef DEBUG
-	    set_client_status(session, "send_to_client()::send()");
-#endif
-
 #ifdef HAVE_SSL
 		if (session->binding->use_ssl) {
 			if ((bytes_sent = ssl_send(session->ssl_data, (char*)(buffer + total_sent), can_send)) <= 0) {
@@ -101,10 +97,6 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 				bytes_sent = -1;
 			}
 
-#ifdef DEBUG
-	    set_client_status(session, NULL);
-#endif
-
 		/* Handle read result
 		 */
 		if (bytes_sent == -1) {
