@@ -58,6 +58,9 @@ static t_host *new_host(void) {
 		host->execute_cgi        = false;
 		host->time_for_cgi       = 5;
 		host->show_index         = false;
+#ifdef HAVE_XSLT
+		host->index_xslt         = NULL;
+#endif
 		host->use_gz_file        = false;
 		host->access_list        = NULL;
 		host->alter_list         = NULL;
@@ -1044,6 +1047,13 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 	} else if (strcmp(key, "showindex") == 0) {
 		if (parse_yesno(value, &(host->show_index)) == 0) {
 			return true;
+#ifdef HAVE_XSLT
+		} else if (*value == '/') {
+			if ((host->index_xslt = strdup(value)) != NULL) {
+				host->show_index = true;
+				return true;
+			}
+#endif
 		}
 	} else if (strcmp(key, "startfile") == 0) {
 		if (valid_start_file(value)) {
