@@ -76,9 +76,6 @@ static void clear_session(t_session *session) {
 #ifdef HAVE_XSLT
 	session->xslt_file = NULL;
 #endif
-#ifdef DEBUG
-	session->status = NULL;
-#endif
 }
 
 /* Initialize a session-record.
@@ -125,9 +122,6 @@ void reset_session(t_session *session) {
 		unlink(session->uploaded_file);
 		free(session->uploaded_file);
 	}
-#ifdef DEBUG
-	sfree(session->status);
-#endif
 	session->headerfields = remove_headerfields(session->headerfields);
 	if (session->directory != NULL) {
 		pthread_mutex_lock(&(session->directory->client_mutex));
@@ -209,6 +203,7 @@ void determine_request_method(t_session *session) {
  */
 int get_target_extension(t_session *session) {
 	char *last_slash;
+
 #ifdef CIFS
 	sfree(session->extension);
 #endif
@@ -242,28 +237,28 @@ int get_homedir(t_session *session, char *username) {
 
 	if (username == NULL) {
 		return 500;
-	} else if ((pwd = getpwnam(username)) != NULL) {
-		len = strlen(pwd->pw_dir);
-		old_root = session->host->website_root;
-		if ((session->host->website_root = (char*)malloc(len + strlen(session->config->user_directory) + 2)) == NULL) {
-			session->host->website_root = old_root;
-			return -1;
-		}
-
-		memcpy(session->host->website_root, pwd->pw_dir, len);
-		*(session->host->website_root + len) = '/';
-		strcpy(session->host->website_root + len + 1, session->config->user_directory);
-		if (register_tempdata(&(session->tempdata), session->host->website_root, tc_data) == -1) {
-			free(session->host->website_root);
-			session->host->website_root = old_root;
-			return -1;
-		}
-		session->host->website_root_len = strlen(session->host->website_root);
-
-		return 200;
+	} else if ((pwd = getpwnam(username)) == NULL) {
+		return 404;
 	}
 
-	return 404;
+	len = strlen(pwd->pw_dir);
+	old_root = session->host->website_root;
+	if ((session->host->website_root = (char*)malloc(len + strlen(session->config->user_directory) + 2)) == NULL) {
+		session->host->website_root = old_root;
+		return -1;
+	}
+
+	memcpy(session->host->website_root, pwd->pw_dir, len);
+	*(session->host->website_root + len) = '/';
+	strcpy(session->host->website_root + len + 1, session->config->user_directory);
+	if (register_tempdata(&(session->tempdata), session->host->website_root, tc_data) == -1) {
+		free(session->host->website_root);
+		session->host->website_root = old_root;
+		return -1;
+	}
+	session->host->website_root_len = strlen(session->host->website_root);
+
+	return 200;
 }
 
 /* Dupliacte the active host-record. The duplicate can now savely be altered
@@ -273,14 +268,14 @@ bool duplicate_host(t_session *session) {
 	t_host *new_host;
 
 	if ((session->host != NULL) && (session->host_copied == false)) {
-		if ((new_host = (t_host*)malloc(sizeof(t_host))) != NULL) {
-			memcpy(new_host, session->host, sizeof(t_host));
-			new_host->next = NULL;
-			session->host = new_host;
-			session->host_copied = true;
-		} else {
+		if ((new_host = (t_host*)malloc(sizeof(t_host))) == NULL) {
 			return false;
 		}
+
+		memcpy(new_host, session->host, sizeof(t_host));
+		new_host->next = NULL;
+		session->host = new_host;
+		session->host_copied = true;
 	}
 
 	return true;
@@ -317,74 +312,70 @@ int copy_directory_settings(t_session *session) {
 				match = (strstr(session->file_on_disk, dir->path) != NULL);
 			}
 			if (match) {
-				if (duplicate_host(session)) {
-					session->directory = dir;
+				session->directory = dir;
 
-					if (dir->max_clients > -1) {
-						pthread_mutex_lock(&(dir->client_mutex));
-						if (dir->nr_of_clients < dir->max_clients) {
-							session->throttle = dir->session_speed = dir->upload_speed / ++dir->nr_of_clients;
-							pthread_mutex_unlock(&(dir->client_mutex));
-							session->part_of_dirspeed = true;
-						} else {
-							pthread_mutex_unlock(&(dir->client_mutex));
-							return 503;
-						}
+				if (dir->max_clients > -1) {
+					pthread_mutex_lock(&(dir->client_mutex));
+					if (dir->nr_of_clients < dir->max_clients) {
+						session->throttle = dir->session_speed = dir->upload_speed / ++dir->nr_of_clients;
+						pthread_mutex_unlock(&(dir->client_mutex));
+						session->part_of_dirspeed = true;
+					} else {
+						pthread_mutex_unlock(&(dir->client_mutex));
+						return 503;
 					}
-					if (dir->wrap_cgi != NULL) {
-						session->host->wrap_cgi = dir->wrap_cgi;
-					}
-					if (dir->start_file != NULL) {
-						session->host->start_file = dir->start_file;
-					}
-					if (dir->execute_cgiset) {
-						session->host->execute_cgi = dir->execute_cgi;
-					}
-					if (dir->show_index_set) {
-						session->host->show_index = dir->show_index;
-					}
-					if (dir->follow_symlinks_set) {
-						session->host->follow_symlinks = dir->follow_symlinks;
-					}
-					if (dir->use_gz_file_set) {
-						session->host->use_gz_file = dir->use_gz_file;
-					}
-					if (dir->access_list != NULL) {
-						session->host->access_list = dir->access_list;
-					}
-					if (dir->alter_list != NULL) {
-						session->host->alter_list = dir->alter_list;
-					}
-					if (dir->alter_fmode != 0) {
-						session->host->alter_fmode = dir->alter_fmode;
-					}
-					if (dir->image_referer.size > 0) {
-						session->host->image_referer.size = dir->image_referer.size;
-						session->host->image_referer.item = dir->image_referer.item;
-						session->host->imgref_replacement = dir->imgref_replacement;
-					}
-					if (dir->passwordfile != NULL) {
-						session->host->auth_method = dir->auth_method;
-						session->host->passwordfile = dir->passwordfile;
-						if (dir->groupfile != NULL) {
-							session->host->groupfile = dir->groupfile;
-						}
-					}
-					if (dir->required_group.size > 0) {
-						session->host->required_group.size = dir->required_group.size;
-						session->host->required_group.item = dir->required_group.item;
-					}
-					if (dir->alter_group.size > 0) {
-						session->host->alter_group.size = dir->alter_group.size;
-						session->host->alter_group.item = dir->alter_group.item;
-					}
-					if (dir->time_for_cgi > -1) {
-						session->host->time_for_cgi = dir->time_for_cgi;
-					}
-					break;
-				} else {
-					return 500;
 				}
+				if (dir->wrap_cgi != NULL) {
+					session->host->wrap_cgi = dir->wrap_cgi;
+				}
+				if (dir->start_file != NULL) {
+					session->host->start_file = dir->start_file;
+				}
+				if (dir->execute_cgiset) {
+					session->host->execute_cgi = dir->execute_cgi;
+				}
+				if (dir->show_index_set) {
+					session->host->show_index = dir->show_index;
+				}
+				if (dir->follow_symlinks_set) {
+					session->host->follow_symlinks = dir->follow_symlinks;
+				}
+				if (dir->use_gz_file_set) {
+					session->host->use_gz_file = dir->use_gz_file;
+				}
+				if (dir->access_list != NULL) {
+					session->host->access_list = dir->access_list;
+				}
+				if (dir->alter_list != NULL) {
+					session->host->alter_list = dir->alter_list;
+				}
+				if (dir->alter_fmode != 0) {
+					session->host->alter_fmode = dir->alter_fmode;
+				}
+				if (dir->image_referer.size > 0) {
+					session->host->image_referer.size = dir->image_referer.size;
+					session->host->image_referer.item = dir->image_referer.item;
+					session->host->imgref_replacement = dir->imgref_replacement;
+				}
+				if (dir->passwordfile != NULL) {
+					session->host->auth_method = dir->auth_method;
+					session->host->passwordfile = dir->passwordfile;
+					if (dir->groupfile != NULL) {
+						session->host->groupfile = dir->groupfile;
+					}
+				}
+				if (dir->required_group.size > 0) {
+					session->host->required_group.size = dir->required_group.size;
+					session->host->required_group.item = dir->required_group.item;
+				}
+				if (dir->alter_group.size > 0) {
+					session->host->alter_group.size = dir->alter_group.size;
+					session->host->alter_group.item = dir->alter_group.item;
+				}
+				if (dir->time_for_cgi > -1) {
+					session->host->time_for_cgi = dir->time_for_cgi;
+				}
+				break;
 			}
 		}
 		dir = dir->next;
@@ -425,7 +416,7 @@ bool client_is_rejected_bot(t_session *session) {
 
 	botlist = session->host->deny_bot;
 	while (botlist != NULL) {
-		if (strstr(useragent, botlist->bot) != NULL) {
+		if (strcasestr(useragent, botlist->bot) != NULL) {
 			for (i = 0; i < botlist->uri.size; i++) {
 				len = strlen(*(botlist->uri.item + i));
 				if (session->uri_len >= len) {
@@ -494,15 +485,3 @@ void close_socket(t_session *session) {
 		session->socket_open = false;
 	}
 }
-
-#ifdef DEBUG
-void set_client_status(t_session *session, char *status) {
-	fprintf(stderr, "Client status: %s\n", status);
-	sfree(session->status);
-	if (status != NULL) {
-		session->status = strdup(status);
-	} else {
-		session->status = NULL;
-	}
-}
-#endif
